@@ -40,7 +40,9 @@ class Console(abc.ABC):
     with them when their input is needed"""
 
     @abc.abstractmethod
-    def start_snapshot_progress(self, snapshot_name: str, total_batches: int) -> None:
+    def start_snapshot_progress(
+        self, snapshot: Snapshot, total_batches: int, environment: str
+    ) -> None:
         """Indicates that a new load progress has begun."""
 
     @abc.abstractmethod
@@ -48,12 +50,20 @@ class Console(abc.ABC):
         """Update snapshot progress."""
 
     @abc.abstractmethod
-    def complete_snapshot_progress(self) -> None:
-        """Indicates that load progress is complete."""
+    def stop_snapshot_progress(self, success: bool = True) -> None:
+        """Stop the load progress"""
 
     @abc.abstractmethod
-    def stop_snapshot_progress(self) -> None:
-        """Stop the load progress"""
+    def start_promotion_progress(self, environment: str, total_tasks: int) -> None:
+        """Indicates that a new promotion progress has begun."""
+
+    @abc.abstractmethod
+    def update_promotion_progress(self, num_tasks: int) -> None:
+        """Update promotion progress."""
+
+    @abc.abstractmethod
+    def stop_promotion_progress(self, success: bool = True) -> None:
+        """Stop the promotion progress"""
 
     @abc.abstractmethod
     def show_model_difference_summary(
@@ -115,8 +125,10 @@ class TerminalConsole(Console):
 
     def __init__(self, console: t.Optional[RichConsole] = None, **kwargs: t.Any) -> None:
         self.console: RichConsole = console or srich.console
-        self.progress: t.Optional[Progress] = None
-        self.tasks: t.Dict[str, t.Tuple[TaskID, int]] = {}
+        self.evaluation_progress: t.Optional[Progress] = None
+        self.evaluation_tasks: t.Dict[str, t.Tuple[TaskID, int]] = {}
+        self.promotion_progress: t.Optional[Progress] = None
+        self.promotion_task: t.Optional[TaskID] = None
         self.loading_status: t.Dict[uuid.UUID, Status] = {}
 
     def _print(self, value: t.Any, **kwargs: t.Any) -> None:
@@ -128,11 +140,13 @@ class TerminalConsole(Console):
     def _confirm(self, message: str, **kwargs: t.Any) -> bool:
         return Confirm.ask(message, console=self.console, **kwargs)
 
-    def start_snapshot_progress(self, snapshot_name: str, total_batches: int) -> None:
+    def start_snapshot_progress(
+        self, snapshot: Snapshot, total_batches: int, environment: str
+    ) -> None:
         """Indicates that a new load progress has begun."""
-        if not self.progress:
-            self.progress = Progress(
-                TextColumn("[bold blue]{task.fields[snapshot_name]}", justify="right"),
+        if not self.evaluation_progress:
+            self.evaluation_progress = Progress(
+                TextColumn("[bold blue]{task.fields[view_name]}", justify="right"),
                 BarColumn(bar_width=40),
                 "[progress.percentage]{task.percentage:>3.1f}%",
                 "•",
@@ -141,12 +155,13 @@ class TerminalConsole(Console):
                 TimeElapsedColumn(),
                 console=self.console,
             )
-            self.progress.start()
-            self.tasks = {}
-        self.tasks[snapshot_name] = (
-            self.progress.add_task(
-                f"Running {snapshot_name}...",
-                snapshot_name=snapshot_name,
+            self.evaluation_progress.start()
+            self.evaluation_tasks = {}
+        view_name = snapshot.qualified_view_name.for_environment(environment)
+        self.evaluation_tasks[snapshot.name] = (
+            self.evaluation_progress.add_task(
+                f"Running {view_name}...",
+                view_name=view_name,
                 total=total_batches,
             ),
             total_batches,
@@ -154,28 +169,49 @@ class TerminalConsole(Console):
 
     def update_snapshot_progress(self, snapshot_name: str, num_batches: int) -> None:
         """Update snapshot progress."""
-        if self.progress and self.tasks:
-            task_id = self.tasks[snapshot_name][0]
-            self.progress.update(task_id, advance=num_batches)
+        if self.evaluation_progress and self.evaluation_tasks:
+            task_id = self.evaluation_tasks[snapshot_name][0]
+            self.evaluation_progress.update(task_id, refresh=True, advance=num_batches)
 
-    def complete_snapshot_progress(self) -> None:
-        """Indicates that load progress is complete"""
-        # Make sure all tasks are marked as completed when we're done processing the batches. The
-        # refresh argument is used so that the bars are finalized properly in Jupyter notebooks.
-        # See first "Note" here: https://rich.readthedocs.io/en/stable/progress.html#progress-display
-        if self.progress:
-            for task_id, total_batches in self.tasks.values():
-                self.progress.update(task_id, refresh=True, completed=total_batches)
-
-        self.log_success("All model batches have been executed successfully")
-        self.stop_snapshot_progress()
-
-    def stop_snapshot_progress(self) -> None:
+    def stop_snapshot_progress(self, success: bool = True) -> None:
         """Stop the load progress"""
-        self.tasks = {}
-        if self.progress:
-            self.progress.stop()
-            self.progress = None
+        self.evaluation_tasks = {}
+        if self.evaluation_progress:
+            self.evaluation_progress.stop()
+            self.evaluation_progress = None
+            if success:
+                self.log_success("All model batches have been executed successfully")
+
+    def start_promotion_progress(self, environment: str, total_tasks: int) -> None:
+        """Indicates that a new promotion progress has begun."""
+        if self.promotion_progress is None:
+            self.promotion_progress = Progress(
+                TextColumn(f"[bold blue]Virtually Updating '{environment}'", justify="right"),
+                BarColumn(bar_width=40),
+                "[progress.percentage]{task.percentage:>3.1f}%",
+                "•",
+                TimeElapsedColumn(),
+                console=self.console,
+            )
+            self.promotion_progress.start()
+            self.promotion_task = self.promotion_progress.add_task(
+                f"Virtually Updating {environment}...",
+                total=total_tasks,
+            )
+
+    def update_promotion_progress(self, num_tasks: int) -> None:
+        """Update promotion progress."""
+        if self.promotion_progress is not None and self.promotion_task is not None:
+            self.promotion_progress.update(self.promotion_task, refresh=True, advance=num_tasks)
+
+    def stop_promotion_progress(self, success: bool = True) -> None:
+        """Stop the promotion progress"""
+        self.promotion_task = None
+        if self.promotion_progress is not None:
+            self.promotion_progress.stop()
+            self.promotion_progress = None
+            if success:
+                self.log_success("The target environment has been updated successfully")
 
     def show_model_difference_summary(
         self, context_diff: ContextDiff, detailed: bool = False
@@ -186,9 +222,19 @@ class TerminalConsole(Console):
             context_diff: The context diff to use to print the summary
             detailed: Show the actual SQL differences if True.
         """
-        if not context_diff.has_differences:
+        if context_diff.is_new_environment:
+            self._print(
+                Tree(
+                    f"[bold]New environment `{context_diff.environment}` will be created from `{context_diff.create_from}`"
+                )
+            )
+            if not context_diff.has_snapshot_changes:
+                return
+
+        if not context_diff.has_changes:
             self._print(Tree(f"[bold]No differences when compared to `{context_diff.environment}`"))
             return
+
         tree = Tree(f"[bold]Summary of differences against `{context_diff.environment}`:")
 
         if context_diff.added:
@@ -246,7 +292,7 @@ class TerminalConsole(Console):
         if plan.requires_backfill:
             self._show_missing_dates(plan)
             self._prompt_backfill(plan, auto_apply)
-        elif plan.context_diff.has_differences and not auto_apply:
+        elif plan.context_diff.has_changes and not auto_apply:
             self._prompt_promote(plan)
 
     def _prompt_categorize(self, plan: Plan, auto_apply: bool) -> None:
@@ -299,31 +345,32 @@ class TerminalConsole(Console):
         is_forward_only_dev = plan.is_dev and plan.forward_only
         backfill_or_preview = "preview" if is_forward_only_dev else "backfill"
 
-        if not plan.override_start:
-            blank_meaning = (
-                "to preview starting from yesterday"
-                if is_forward_only_dev
-                else "for the beginning of history"
-            )
-            start = self._prompt(
-                f"Enter the {backfill_or_preview} start date (eg. '1 year', '2020-01-01') or blank {blank_meaning}",
-            )
-            if start:
-                plan.start = start
+        if plan.is_start_and_end_allowed:
+            if not plan.override_start:
+                blank_meaning = (
+                    "to preview starting from yesterday"
+                    if is_forward_only_dev
+                    else "for the beginning of history"
+                )
+                start = self._prompt(
+                    f"Enter the {backfill_or_preview} start date (eg. '1 year', '2020-01-01') or blank {blank_meaning}",
+                )
+                if start:
+                    plan.start = start
 
-        if plan.is_end_allowed and not plan.override_end:
-            end = self._prompt(
-                f"Enter the {backfill_or_preview} end date (eg. '1 month ago', '2020-01-01') or blank to {backfill_or_preview} up until now",
-            )
-            if end:
-                plan.end = end
+            if not plan.override_end:
+                end = self._prompt(
+                    f"Enter the {backfill_or_preview} end date (eg. '1 month ago', '2020-01-01') or blank to {backfill_or_preview} up until now",
+                )
+                if end:
+                    plan.end = end
 
         if not auto_apply and self._confirm(f"Apply - {backfill_or_preview.capitalize()} Tables"):
             plan.apply()
 
     def _prompt_promote(self, plan: Plan) -> None:
         if self._confirm(
-            f"Apply - Logical Update",
+            f"Apply - Virtual Update",
         ):
             plan.apply()
 
@@ -464,7 +511,7 @@ class NotebookMagicConsole(TerminalConsole):
         import ipywidgets as widgets
 
         button = widgets.Button(
-            description="Apply - Logical Update",
+            description="Apply - Virtual Update",
             disabled=False,
             button_style="success",
             # Auto will make the button really large.
@@ -517,17 +564,19 @@ class NotebookMagicConsole(TerminalConsole):
             plan.end = change["new"]
             self._show_options_after_categorization(plan, auto_apply)
 
-        add_to_layout_widget(
-            prompt,
-            widgets.HBox(
-                [
-                    widgets.Label(f"Start {backfill_or_preview} Date:", layout={"width": "8rem"}),
-                    _date_picker(plan, to_date(plan.start), start_change_callback),
-                ]
-            ),
-        )
+        if plan.is_start_and_end_allowed:
+            add_to_layout_widget(
+                prompt,
+                widgets.HBox(
+                    [
+                        widgets.Label(
+                            f"Start {backfill_or_preview} Date:", layout={"width": "8rem"}
+                        ),
+                        _date_picker(plan, to_date(plan.start), start_change_callback),
+                    ]
+                ),
+            )
 
-        if plan.is_end_allowed:
             add_to_layout_widget(
                 prompt,
                 widgets.HBox(
@@ -664,7 +713,7 @@ class DatabricksMagicConsole(TerminalConsole):
         return super()._prompt("", **kwargs)
 
     def _confirm(self, message: str, **kwargs: t.Any) -> bool:
-        message = f"{message} \[y/n]"
+        message = f"{message} [y/n]"
         self._print(message)
         return super()._confirm("", **kwargs)
 
