@@ -42,7 +42,6 @@ from types import MappingProxyType
 
 import pandas as pd
 from sqlglot import exp
-from sqlglot.executor import execute
 
 from sqlmesh.core import constants as c
 from sqlmesh.core._typing import NotificationTarget
@@ -56,18 +55,17 @@ from sqlmesh.core.environment import Environment
 from sqlmesh.core.hooks import hook
 from sqlmesh.core.loader import Loader, SqlMeshLoader, update_model_schemas
 from sqlmesh.core.macros import ExecutableOrMacro
-from sqlmesh.core.model import Model, SqlModel
+from sqlmesh.core.model import Model
 from sqlmesh.core.plan import Plan
 from sqlmesh.core.scheduler import Scheduler
 from sqlmesh.core.snapshot import (
     Snapshot,
     SnapshotEvaluator,
     SnapshotFingerprint,
-    categorize_change,
     to_table_mapping,
 )
 from sqlmesh.core.state_sync import StateReader, StateSync
-from sqlmesh.core.test import run_all_model_tests
+from sqlmesh.core.test import run_all_model_tests, run_model_tests
 from sqlmesh.core.user import User
 from sqlmesh.utils import UniqueKeyDict, sys_path
 from sqlmesh.utils.dag import DAG
@@ -273,8 +271,10 @@ class Context(BaseContext):
         if isinstance(model, str):
             model = self._models[model]
 
+        path = model._path  # type: ignore
         # model.copy() can't be used here due to a cached state that can be a part of a model instance.
         model = t.cast(Model, type(model)(**{**t.cast(Model, model).dict(), **kwargs}))
+        model._path = path
         self._models.update({model.name: model})
 
         self._add_model_to_dag(model)
@@ -282,19 +282,22 @@ class Context(BaseContext):
 
         return model
 
-    def scheduler(self, global_state: bool = False) -> Scheduler:
+    def scheduler(self, environment: t.Optional[str] = None) -> Scheduler:
         """Returns the built-in scheduler.
 
         Args:
-            global_state: Whether to initialize the scheduler from the persisted state
-                or from the currently loaded local state. Default: False.
+            environment: The target environment to source model snapshots from, or None
+                if snapshots should be sourced from the currently loaded local state.
 
         Returns:
             The built-in scheduler instance.
         """
         snapshots: t.Iterable[Snapshot]
-        if global_state:
-            snapshots = self.state_sync.get_snapshots(None).values()
+        if environment is not None:
+            stored_environment = self.state_sync.get_environment(environment)
+            if stored_environment is None:
+                raise ConfigError(f"Environment '{environment}' was not found.")
+            snapshots = self.state_sync.get_snapshots(stored_environment.snapshots).values()
         else:
             snapshots = self.snapshots.values()
 
@@ -319,7 +322,9 @@ class Context(BaseContext):
                 raise ConfigError(
                     "The operation is not supported when using a read-only state sync"
                 )
-            self._state_sync.init_schema()
+
+            if self._state_sync.get_versions(validate=False).schema_version == 0:
+                self._state_sync.migrate()
         return self._state_sync
 
     @property
@@ -386,21 +391,27 @@ class Context(BaseContext):
 
     def run(
         self,
+        environment: t.Optional[str] = None,
+        *,
         start: t.Optional[TimeLike] = None,
         end: t.Optional[TimeLike] = None,
         latest: t.Optional[TimeLike] = None,
-        global_state: bool = False,
+        skip_janitor: bool = False,
     ) -> None:
         """Run the entire dag through the scheduler.
 
         Args:
+            environment: The target environment to source model snapshots from and virtually update. Default: prod.
             start: The start of the interval to render.
             end: The end of the interval to render.
             latest: The latest time used for non incremental datasets.
-            global_state: If set to True runs against the persisted state,
-                otherwise uses the currently loaded local state. Default: False.
+            skip_janitor: Whether to skip the janitor task.
         """
-        return self.scheduler(global_state).run(start, end, latest)
+        environment = environment or c.PROD
+        self.scheduler(environment=environment).run(environment, start, end, latest)
+
+        if not skip_janitor:
+            self._run_janitor()
 
     def get_model(self, name: str) -> t.Optional[Model]:
         """Returns a model with the given name or None if a model with such name doesn't exist."""
@@ -423,7 +434,24 @@ class Context(BaseContext):
 
     @property
     def snapshots(self) -> t.Dict[str, Snapshot]:
-        """Generates and returns snapshots based on models registered in this context."""
+        """Generates and returns snapshots based on models registered in this context.
+
+        If one of the snapshots has been previosly stored in the persisted state, the stored
+        instance will be returned.
+        """
+        local_snapshots = self.local_snapshots
+
+        stored_snapshots = self.state_reader.get_snapshots(
+            [s.snapshot_id for s in local_snapshots.values()]
+        )
+
+        return {name: stored_snapshots.get(s.snapshot_id, s) for name, s in local_snapshots.items()}
+
+    @property
+    def local_snapshots(self) -> t.Dict[str, Snapshot]:
+        """Generates and returns snapshots based on models registered in this context without reconciling them
+        with the persisted state.
+        """
         local_snapshots = {}
         fingerprint_cache: t.Dict[str, SnapshotFingerprint] = {}
         for model in self._models.values():
@@ -436,25 +464,7 @@ class Context(BaseContext):
                 cache=fingerprint_cache,
             )
             local_snapshots[model.name] = snapshot
-
-        stored_snapshots = self.state_reader.get_snapshots(
-            [s.snapshot_id for s in local_snapshots.values()]
-        )
-
-        return {name: stored_snapshots.get(s.snapshot_id, s) for name, s in local_snapshots.items()}
-
-    @property
-    def _model_tables(self) -> t.Dict[str, str]:
-        """Mapping of model name to physical table name.
-
-        If a snapshot has not been versioned yet, its view name will be returned.
-        """
-        return {
-            name: snapshot.table_name()
-            if snapshot.version
-            else snapshot.qualified_view_name.for_environment(c.PROD)
-            for name, snapshot in self.snapshots.items()
-        }
+        return local_snapshots
 
     def render(
         self,
@@ -522,10 +532,7 @@ class Context(BaseContext):
             start: The start of the interval to evaluate.
             end: The end of the interval to evaluate.
             latest: The latest time used for non incremental datasets.
-            limit: A limit applied to the model, this must be > 0. If this argument is omitted
-                and the model contains a LIMIT clause, then the clause's expression will be used
-                instead. In any case, the final limit is bounded by the DEFAULT_MAX_LIMIT constant.
-                Default: DEFAULT_MAX_LIMIT
+            limit: A limit applied to the model.
         """
         if isinstance(model_or_snapshot, str):
             snapshot = self.snapshots[model_or_snapshot]
@@ -534,25 +541,18 @@ class Context(BaseContext):
         else:
             snapshot = self.snapshots[model_or_snapshot.name]
 
-        if not limit or limit <= 0:
-            limit = c.DEFAULT_MAX_LIMIT
-            if isinstance(snapshot.model, SqlModel):
-                model_limit = snapshot.model.render_query().args.get("limit")
-
-                if model_limit:
-                    limit = min(limit, execute(exp.select(model_limit.expression)).rows[0][0])
-
         df = self.snapshot_evaluator.evaluate(
             snapshot,
             start,
             end,
             latest,
             snapshots=self.snapshots,
-            limit=t.cast(int, limit),
+            limit=limit or c.DEFAULT_MAX_LIMIT,
         )
 
         if df is None:
             raise RuntimeError(f"Error evaluating {snapshot.model.name}")
+
         return df
 
     def format(self) -> None:
@@ -566,26 +566,13 @@ class Context(BaseContext):
                 file.write(format_model_expressions(expressions, model.dialect))
                 file.truncate()
 
-    def _run_plan_tests(
-        self, skip_tests: bool = False
-    ) -> t.Tuple[t.Optional[unittest.result.TestResult], t.Optional[str]]:
-        if self._test_engine_adapter and not skip_tests:
-            result, test_output = self.run_tests()
-            self.console.log_test_results(result, test_output, self._test_engine_adapter.dialect)
-            if not result.wasSuccessful():
-                raise PlanError(
-                    "Cannot generate plan due to failing test(s). Fix test(s) and run again"
-                )
-            return result, test_output
-        return None, None
-
     def plan(
         self,
         environment: t.Optional[str] = None,
         *,
         start: t.Optional[TimeLike] = None,
         end: t.Optional[TimeLike] = None,
-        from_: t.Optional[str] = None,
+        create_from: t.Optional[str] = None,
         skip_tests: bool = False,
         restate_models: t.Optional[t.Iterable[str]] = None,
         no_gaps: bool = False,
@@ -604,7 +591,8 @@ class Context(BaseContext):
             environment: The environment to diff and plan against.
             start: The start date of the backfill if there is one.
             end: The end date of the backfill if there is one.
-            from_: The environment to base the plan on instead of local files.
+            create_from: The environment to create the target environment from if it
+                doesn't exist. If not specified, the "prod" environment will be used.
             skip_tests: Unit tests are run by default so this will skip them if enabled
             restate_models: A list of of either internal or external models that need to be restated
                 for the given plan interval. If the target environment is a production environment,
@@ -638,22 +626,8 @@ class Context(BaseContext):
 
         self._run_plan_tests(skip_tests)
 
-        if from_:
-            env = self.state_reader.get_environment(from_)
-            if not env:
-                raise PlanError(f"Environment '{from_}' not found.")
-
-            start = start or env.start_at
-            end = end or env.end_at
-            snapshots = {
-                snapshot.name: snapshot
-                for snapshot in self.state_reader.get_snapshots(env.snapshots).values()
-            }
-        else:
-            snapshots = None
-
         plan = Plan(
-            context_diff=self._context_diff(environment or c.PROD, snapshots),
+            context_diff=self._context_diff(environment or c.PROD, create_from=create_from),
             dag=self.dag,
             state_reader=self.state_reader,
             start=start,
@@ -665,22 +639,10 @@ class Context(BaseContext):
             is_dev=environment != c.PROD,
             forward_only=forward_only,
             environment_ttl=self.config.environment_ttl,
+            categorizer_config=self.config.auto_categorize_changes,
+            auto_categorization_enabled=not no_auto_categorization,
         )
 
-        auto_categorize_changes = (
-            self.config.auto_categorize_changes
-            if no_auto_categorization is None
-            else not no_auto_categorization
-        )
-
-        if auto_categorize_changes and not forward_only:
-            # Attempt to automatically determine and assign change categories.
-            for new, old in plan.context_diff.modified_snapshots.values():
-                if plan.is_new_snapshot(new):
-                    change_category = categorize_change(new, old)
-                    if change_category is not None:
-                        plan.set_choice(new, change_category)
-
         if not no_prompts:
             self.console.plan(plan, auto_apply)
         elif auto_apply:
@@ -697,7 +659,7 @@ class Context(BaseContext):
         Args:
             plan: The plan to apply.
         """
-        if not plan.context_diff.has_differences and not plan.requires_backfill:
+        if not plan.context_diff.has_changes and not plan.requires_backfill:
             return
         if plan.uncategorized:
             raise PlanError("Can't apply a plan with uncategorized changes.")
@@ -768,20 +730,36 @@ class Context(BaseContext):
                 "Graphviz is pip-installed but the system install is missing. Instructions: https://graphviz.org/download/"
             ) from e
 
-    def run_tests(self, path: t.Optional[str] = None) -> t.Tuple[unittest.result.TestResult, str]:
+    def test(
+        self,
+        match_patterns: t.Optional[t.List[str]] = None,
+        tests: t.Optional[t.List[str]] = None,
+        verbose: bool = False,
+    ) -> unittest.result.TestResult:
         """Discover and run model tests"""
-        test_output = StringIO()
-        with contextlib.redirect_stderr(test_output):
-            try:
-                result = run_all_model_tests(
-                    path=Path(path) if path else self.test_directory_path,
-                    snapshots=self.snapshots,
+        verbosity = 2 if verbose else 1
+        try:
+            if tests:
+                result = run_model_tests(
+                    tests=tests,
+                    snapshots=self.local_snapshots,
                     engine_adapter=self._test_engine_adapter,
+                    verbosity=verbosity,
+                    patterns=match_patterns,
                     ignore_patterns=self.ignore_patterns,
                 )
-            finally:
-                self._test_engine_adapter.close()
-        return result, test_output.getvalue()
+            else:
+                result = run_all_model_tests(
+                    path=self.test_directory_path,
+                    snapshots=self.local_snapshots,
+                    engine_adapter=self._test_engine_adapter,
+                    verbosity=verbosity,
+                    patterns=match_patterns,
+                    ignore_patterns=self.ignore_patterns,
+                )
+        finally:
+            self._test_engine_adapter.close()
+        return result
 
     def audit(
         self,
@@ -831,18 +809,56 @@ class Context(BaseContext):
             self.console.show_sql(f"{error.query}")
         self.console.log_status_update("Done.")
 
+    def migrate(self) -> None:
+        """Migrates SQLMesh to the current running version.
+
+        Please contact your SQLMesh administrator before doing this.
+        """
+        self.state_sync.migrate()
+
     def close(self) -> None:
         """Releases all resources allocated by this context."""
         self.snapshot_evaluator.close()
 
+    def _run_plan_tests(
+        self, skip_tests: bool = False
+    ) -> t.Tuple[t.Optional[unittest.result.TestResult], t.Optional[str]]:
+        if self._test_engine_adapter and not skip_tests:
+            test_output_io = StringIO()
+            with contextlib.redirect_stderr(test_output_io):
+                result = self.test()
+            test_output = test_output_io.getvalue()
+            self.console.log_test_results(result, test_output, self._test_engine_adapter.dialect)
+            if not result.wasSuccessful():
+                raise PlanError(
+                    "Cannot generate plan due to failing test(s). Fix test(s) and run again"
+                )
+            return result, test_output
+        return None, None
+
+    @property
+    def _model_tables(self) -> t.Dict[str, str]:
+        """Mapping of model name to physical table name.
+
+        If a snapshot has not been versioned yet, its view name will be returned.
+        """
+        return {
+            name: snapshot.table_name()
+            if snapshot.version
+            else snapshot.qualified_view_name.for_environment(c.PROD)
+            for name, snapshot in self.snapshots.items()
+        }
+
     def _context_diff(
         self,
         environment: str | Environment,
         snapshots: t.Optional[t.Dict[str, Snapshot]] = None,
+        create_from: t.Optional[str] = None,
     ) -> ContextDiff:
-        environment = environment or c.PROD
         environment = Environment.normalize_name(environment)
-        return ContextDiff.create(environment, snapshots or self.snapshots, self.state_reader)
+        return ContextDiff.create(
+            environment, snapshots or self.snapshots, create_from or c.PROD, self.state_reader
+        )
 
     def _load_config(self, config: t.Union[str, Config]) -> Config:
         if isinstance(config, Config):
@@ -857,26 +873,15 @@ class Context(BaseContext):
         ]
         return load_config_from_paths(*lookup_paths, config_name=config)
 
-    def glob_path(self, path: Path, file_extension: str) -> t.Generator[Path, None, None]:
-        """
-        Globs the provided path for the file extension but also removes any filepaths that match an ignore
-        pattern either set in constants or provided in config
-
-        Args:
-            path: The filepath to glob
-            file_extension: The extension to check for in that path (checks recursively in zero or more subdirectories)
-
-        Returns:
-            Matched paths that are not ignored
-        """
-        for filepath in path.glob(f"**/*{file_extension}"):
-            for ignore_pattern in self.ignore_patterns:
-                if filepath.match(ignore_pattern):
-                    break
-            else:
-                yield filepath
-
     def _add_model_to_dag(self, model: Model) -> None:
         self.dag.graph[model.name] = set()
 
         self.dag.add(model.name, model.depends_on)
+
+    def _run_janitor(self) -> None:
+        expired_environments = self.state_sync.delete_expired_environments()
+        for expired_environment in expired_environments:
+            self.snapshot_evaluator.demote(expired_environment.snapshots, expired_environment.name)
+
+        expired_snapshots = self.state_sync.delete_expired_snapshots()
+        self.snapshot_evaluator.cleanup(expired_snapshots)
