@@ -9,6 +9,7 @@ from sqlmesh.core.engine_adapter.bigquery import BigQueryEngineAdapter
 from sqlmesh.core.engine_adapter.databricks import DatabricksSparkSessionEngineAdapter
 from sqlmesh.core.engine_adapter.databricks_api import DatabricksSQLEngineAdapter
 from sqlmesh.core.engine_adapter.duckdb import DuckDBEngineAdapter
+from sqlmesh.core.engine_adapter.postgres import PostgresEngineAdapter
 from sqlmesh.core.engine_adapter.redshift import RedshiftEngineAdapter
 from sqlmesh.core.engine_adapter.shared import TransactionType
 from sqlmesh.core.engine_adapter.snowflake import SnowflakeEngineAdapter
@@ -21,7 +22,7 @@ DIALECT_TO_ENGINE_ADAPTER = {
     "snowflake": SnowflakeEngineAdapter,
     "databricks": DatabricksSparkSessionEngineAdapter,
     "redshift": RedshiftEngineAdapter,
-    "postgres": EngineAdapterWithIndexSupport,
+    "postgres": PostgresEngineAdapter,
     "mysql": EngineAdapterWithIndexSupport,
     "mssql": EngineAdapterWithIndexSupport,
 }
