@@ -8,6 +8,7 @@ from sqlmesh.core import scheduler
 from sqlmesh.core.config import CategorizerConfig
 from sqlmesh.core.context_diff import ContextDiff
 from sqlmesh.core.environment import Environment
+from sqlmesh.core.model.meta import IntervalUnit
 from sqlmesh.core.snapshot import (
     Intervals,
     Snapshot,
@@ -22,7 +23,9 @@ from sqlmesh.utils.dag import DAG
 from sqlmesh.utils.date import (
     TimeLike,
     make_inclusive,
+    make_inclusive_end,
     now,
+    to_date,
     to_ds,
     to_timestamp,
     validate_date_range,
@@ -139,6 +142,8 @@ class Plan:
     @property
     def start(self) -> TimeLike:
         """Returns the start of the plan or the earliest date of all snapshots."""
+        if not self.override_start and not self._missing_intervals:
+            return scheduler.earliest_start_date(self.snapshots)
         return self._start or (
             min(
                 start
@@ -153,20 +158,48 @@ class Plan:
     def start(self, new_start: TimeLike) -> None:
         self._ensure_valid_date_range(new_start, self._end)
         self.set_start(new_start)
+        self.override_start = True
 
     def set_start(self, new_start: TimeLike) -> None:
         self._start = new_start
         self.__missing_intervals = None
 
+    def _get_end_date(self, end_and_units: t.List[t.Tuple[int, IntervalUnit]]) -> TimeLike:
+        if end_and_units:
+            end, unit = max(end_and_units)
+
+            if unit == IntervalUnit.DAY:
+                return to_date(make_inclusive_end(end))
+            return end
+        return now()
+
     @property
     def end(self) -> TimeLike:
         """Returns the end of the plan or now."""
-        return self._end or now()
+        if not self._end or not self.override_end:
+            if self._missing_intervals:
+                return self._get_end_date(
+                    [
+                        (end, snapshot.model.interval_unit())
+                        for snapshot in self.snapshots
+                        if snapshot.version_get_or_generate() in self._missing_intervals
+                        for _, end in self._missing_intervals[snapshot.version_get_or_generate()]
+                    ]
+                )
+            return self._get_end_date(
+                [
+                    (snapshot.intervals[-1][1], snapshot.model.interval_unit())
+                    for snapshot in self.snapshots
+                    if snapshot.intervals
+                ]
+            )
+        return self._end
 
     @end.setter
     def end(self, new_end: TimeLike) -> None:
         self._ensure_valid_date_range(self._start, new_end)
         self._end = new_end
+        self.override_end = True
         self.__missing_intervals = None
 
     @property
@@ -341,7 +374,7 @@ class Plan:
                 else []
             )
 
-            end = self.end
+            end = self._end or now()
             self.__missing_intervals = {
                 snapshot.version_get_or_generate(): missing
                 for snapshot, missing in self._state_reader.missing_intervals(
