@@ -20,7 +20,7 @@ from sqlmesh.core._typing import NotificationTarget
 from sqlmesh.core.console import Console, get_console
 from sqlmesh.core.plan.definition import Plan
 from sqlmesh.core.scheduler import Scheduler
-from sqlmesh.core.snapshot import SnapshotEvaluator
+from sqlmesh.core.snapshot import SnapshotEvaluator, SnapshotInfoLike
 from sqlmesh.core.state_sync import StateSync
 from sqlmesh.core.user import User
 from sqlmesh.schedulers.airflow import common as airflow_common
@@ -73,12 +73,14 @@ class BuiltInPlanEvaluator(PlanEvaluator):
                 max_workers=self.backfill_concurrent_tasks,
                 console=self.console,
             )
-            scheduler.run(plan.start, plan.end, is_dev=plan.is_dev)
+            is_run_successful = scheduler.run(plan.environment.name, plan.start, plan.end)
+            if not is_run_successful:
+                raise SQLMeshError("Plan application failed.")
 
         self._promote(plan)
 
         if not plan.requires_backfill:
-            self.console.log_success("Logical Update executed successfully")
+            self.console.log_success("Virtual Update executed successfully")
 
     def _push(self, plan: Plan) -> None:
         """
@@ -112,20 +114,32 @@ class BuiltInPlanEvaluator(PlanEvaluator):
 
         added, removed = self.state_sync.promote(environment, no_gaps=plan.no_gaps)
 
+        self.console.start_promotion_progress(environment.name, len(added) + len(removed))
+
         if not environment.end_at:
             if not plan.is_dev:
                 self.snapshot_evaluator.migrate(plan.environment.snapshots)
             self.state_sync.unpause_snapshots(added, now())
 
-        self.snapshot_evaluator.promote(
-            added,
-            environment=environment.name,
-            is_dev=plan.is_dev,
-        )
-        self.snapshot_evaluator.demote(
-            removed,
-            environment=environment.name,
-        )
+        def on_complete(snapshot: SnapshotInfoLike) -> None:
+            self.console.update_promotion_progress(1)
+
+        completed = False
+        try:
+            self.snapshot_evaluator.promote(
+                added,
+                environment=environment.name,
+                is_dev=plan.is_dev,
+                on_complete=on_complete,
+            )
+            self.snapshot_evaluator.demote(
+                removed,
+                environment=environment.name,
+                on_complete=on_complete,
+            )
+            completed = True
+        finally:
+            self.console.stop_promotion_progress(success=completed)
 
     def _restate(self, plan: Plan) -> None:
         all_snapshots = (
@@ -211,4 +225,4 @@ class AirflowPlanEvaluator(PlanEvaluator):
                 self.dag_run_poll_interval_secs,
             )
             if not plan_application_succeeded:
-                raise SQLMeshError("Plan application failed")
+                raise SQLMeshError("Plan application failed.")
