@@ -21,7 +21,7 @@ class SqlmeshAirflowPlugin(AirflowPlugin):
     def on_load(cls, *args: t.Any, **kwargs: t.Any) -> None:
         with util.scoped_state_sync() as state_sync:
             try:
-                state_sync.init_schema()
+                state_sync.migrate()
             except Exception as ex:
                 # This method is called once for each Gunicorn worker spawned by the Airflow Webserver,
                 # which leads to SQLMesh schema being initialized concurrently from multiple processes.
@@ -30,4 +30,4 @@ class SqlmeshAirflowPlugin(AirflowPlugin):
                 # the schema initialization once as a workaround.
                 logger.warning("Failed to initialize the SQLMesh State Sync: %s. Retrying...", ex)
                 time.sleep(1)
-                state_sync.init_schema()
+                state_sync.migrate()
