@@ -6,18 +6,19 @@ from collections import OrderedDict
 from os import getenv
 from pathlib import Path
 
-from jinja2 import Environment
 from ruamel.yaml import YAML, CommentedMap
 
 from sqlmesh.utils.errors import SQLMeshError
+from sqlmesh.utils.jinja import ENVIRONMENT
 
-yaml = YAML()
-yaml_env = Environment()
-
-yaml_env.globals.update(env_var=lambda key, default=None: getenv(key, default))
+JINJA_METHODS = {
+    "env_var": lambda key, default=None: getenv(key, default),
+}
 
 
-def load(source: str | Path, raise_if_empty: bool = True) -> t.OrderedDict:
+def load(
+    source: str | Path, raise_if_empty: bool = True, render_jinja: bool = True
+) -> t.OrderedDict:
     """Loads a YAML object from either a raw string or a file."""
     path: t.Optional[Path] = None
 
@@ -26,7 +27,10 @@ def load(source: str | Path, raise_if_empty: bool = True) -> t.OrderedDict:
         with open(source, "r", encoding="utf-8") as file:
             source = file.read()
 
-    contents = yaml.load(yaml_env.from_string(source).render())
+    if render_jinja:
+        source = ENVIRONMENT.from_string(source).render(JINJA_METHODS)
+
+    contents = YAML().load(source)
     if contents is None:
         if raise_if_empty:
             error_path = f" '{path}'" if path else ""
@@ -39,5 +43,5 @@ def load(source: str | Path, raise_if_empty: bool = True) -> t.OrderedDict:
 def dumps(value: CommentedMap | OrderedDict) -> str:
     """Dumps a ruamel.yaml loaded object and converts it into a string"""
     result = io.StringIO()
-    yaml.dump(value, result)
+    YAML().dump(value, result)
     return result.getvalue()
