@@ -10,9 +10,9 @@ from sqlmesh.core.config import AutoCategorizationMode, CategorizerConfig
 from sqlmesh.core.model import (
     IncrementalByTimeRangeKind,
     Model,
-    ModelKind,
-    ModelKindName,
+    Seed,
     SeedKind,
+    SeedModel,
     SqlModel,
     create_seed_model,
     load_model,
@@ -53,20 +53,6 @@ def model():
     )
 
 
-@pytest.fixture
-def full_refresh_model():
-    return SqlModel(
-        name="fr_model",
-        kind=ModelKind(name=ModelKindName.FULL),
-        owner="owner",
-        dialect="spark",
-        cron="1 0 * * *",
-        batch_size=30,
-        start="2020-01-01",
-        query=parse_one("SELECT @EACH([1, 2], x -> x), ds FROM parent.tbl"),
-    )
-
-
 @pytest.fixture
 def snapshot(
     model: Model,
@@ -86,25 +72,6 @@ def snapshot(
     return snapshot
 
 
-@pytest.fixture
-def full_refresh_snapshot(
-    full_refresh_model: Model,
-    parent_model: Model,
-    monkeypatch: MonkeyPatch,
-    mocker: MockerFixture,
-    make_snapshot,
-):
-    mock = mocker.Mock()
-    mock.return_value = to_datetime("2022-09-23T00:12:53+00:00")
-    monkeypatch.setattr("sqlmesh.utils.date.now", mock)
-    full_refresh_snapshot = make_snapshot(
-        full_refresh_model,
-        models={parent_model.name: parent_model, full_refresh_model.name: full_refresh_model},
-    )
-    full_refresh_snapshot.version = full_refresh_snapshot.fingerprint
-    return full_refresh_snapshot
-
-
 def test_json(snapshot: Snapshot):
     assert json.loads(snapshot.json()) == {
         "created_ts": 1663891973000,
@@ -242,32 +209,23 @@ def test_missing_intervals(snapshot: Snapshot):
     ]
 
 
-def test_missing_interval_latest(
-    full_refresh_snapshot: Snapshot, monkeypatch: MonkeyPatch, mocker: MockerFixture
-):
-    mock = mocker.Mock()
-    mock.return_value = to_datetime("2020-01-05T00:12:53+00:00")
-    monkeypatch.setattr("sqlmesh.core.snapshot.definition.now", mock)
-    full_refresh_snapshot.add_interval("2020-01-01", "2020-01-01")
-    assert full_refresh_snapshot.missing_intervals("2020-01-01", "2020-01-01", "2020-01-01") == []
-    assert full_refresh_snapshot.missing_intervals("2020-01-02", "2020-01-02", "2020-01-02") == [
+def test_seed_intervals(make_snapshot):
+    snapshot_a = make_snapshot(
+        SeedModel(
+            name="a",
+            kind=SeedKind(path="./path/to/seed"),
+            seed=Seed(content="content"),
+            depends_on=set(),
+        )
+    )
+
+    assert snapshot_a.missing_intervals("2020-01-01", "2020-01-01") == [
+        (to_timestamp("2020-01-01"), to_timestamp("2020-01-02"))
+    ]
+    snapshot_a.add_interval("2020-01-01", "2020-01-01")
+    assert snapshot_a.missing_intervals("2020-01-02", "2020-01-02") == [
         (to_timestamp("2020-01-02"), to_timestamp("2020-01-03"))
     ]
-    assert full_refresh_snapshot.missing_intervals("2020-01-02", "2020-01-03", "2020-01-03") == [
-        (to_timestamp("2020-01-03"), to_timestamp("2020-01-04"))
-    ]
-    assert full_refresh_snapshot.missing_intervals("2020-01-02", "2020-01-03", "2020-01-04") == [
-        (to_timestamp("2020-01-04"), to_timestamp("2020-01-05"))
-    ]
-    assert full_refresh_snapshot.missing_intervals(
-        "2020-01-02", "2020-01-03", "2020-01-03 01:00:00"
-    ) == [(to_timestamp("2020-01-02"), to_timestamp("2020-01-03"))]
-    assert full_refresh_snapshot.missing_intervals(
-        "2020-01-02", "2020-01-03", "2020-01-04 23:59:59"
-    ) == [(to_timestamp("2020-01-03"), to_timestamp("2020-01-04"))]
-    assert full_refresh_snapshot.missing_intervals("2020-01-02", "2020-01-03") == [
-        (to_timestamp("2020-01-04"), to_timestamp("2020-01-05"))
-    ]
 
 
 def test_remove_intervals(snapshot: Snapshot):
@@ -326,7 +284,7 @@ def test_fingerprint(model: Model, parent_model: Model):
 
     original_fingerprint = SnapshotFingerprint(
         data_hash="3042895307",
-        metadata_hash="3589467163",
+        metadata_hash="2417444816",
     )
 
     assert fingerprint == original_fingerprint
@@ -405,7 +363,7 @@ def test_fingerprint_jinja_macros(model: Model):
 
     original_fingerprint = SnapshotFingerprint(
         data_hash="2665680291",
-        metadata_hash="3589467163",
+        metadata_hash="2417444816",
     )
 
     fingerprint = fingerprint_from_model(model, models={})
