@@ -77,26 +77,26 @@ export function toDate(value?: string | number): Date | undefined {
 export function toDateFormat(
   date?: Date,
   format: string = 'yyyy-mm-dd',
+  isUTC: boolean = true,
 ): string {
   if (date == null) return ''
 
-  const year = date.getFullYear()
-  const month = date.getMonth() + 1
-  const day = date.getDate()
-  const hour = date.getHours()
-  const minute = date.getMinutes()
-  const second = date.getSeconds()
+  const year = isUTC ? date.getUTCFullYear() : date.getFullYear()
+  const month = toFormatted(
+    isUTC ? date.getUTCMonth() + 1 : date.getMonth() + 1,
+  )
+  const day = toFormatted(isUTC ? date.getUTCDate() : date.getDate())
+  const hour = toFormatted(isUTC ? date.getUTCHours() : date.getHours())
+  const minute = toFormatted(isUTC ? date.getUTCMinutes() : date.getMinutes())
+  const second = toFormatted(isUTC ? date.getUTCSeconds() : date.getSeconds())
 
-  if (format === 'yyyy-mm-dd')
-    return `${year}-${toFormatted(month)}-${toFormatted(day)}`
-  if (format === 'yyyy-mm-dd hh-mm-ss')
-    return `${year}-${toFormatted(month)}-${toFormatted(day)} ${toFormatted(
-      hour,
-    )}:${toFormatted(minute)}:${toFormatted(second)}`
-  if (format === 'mm/dd/yyyy')
-    return `${toFormatted(month)}/${toFormatted(day)}/${year}`
+  const formats: Record<string, string> = {
+    'mm/dd/yyyy': `${month}/${day}/${year}`,
+    'yyyy-mm-dd': `${year}-${month}-${day}`,
+    'yyyy-mm-dd hh-mm-ss': `${year}-${month}-${day} ${hour}:${minute}:${second}`,
+  }
 
-  return date.toDateString()
+  return formats[format] ?? date.toDateString()
 
   function toFormatted(n: number): string {
     return n.toString().padStart(2, '0')
@@ -129,36 +129,78 @@ export function parseJSON<T>(value: string | null): Optional<T> {
   }
 }
 
+export function debounceSync(
+  fn: (...args: any) => void,
+  delay: number = 500,
+  immediate: boolean = false,
+): (...args: any) => void {
+  let timeoutID: ReturnType<typeof setTimeout> | undefined
+
+  return function callback(...args: any) {
+    const callNow = immediate && timeoutID == null
+
+    clearTimeout(timeoutID)
+
+    timeoutID = setTimeout(() => {
+      timeoutID = undefined
+
+      if (isFalse(immediate)) {
+        fn(...args)
+      }
+    }, delay)
+
+    if (callNow) {
+      fn(...args)
+    }
+  }
+}
+
 export function debounceAsync<T = any>(
   fn: (...args: any) => Promise<T>,
   delay: number = 0,
   immediate: boolean = false,
 ): ((...args: any) => Promise<T>) & { cancel: () => void } {
-  let timeoutID: ReturnType<typeof setTimeout> | undefined
+  let timeoutIdLeading: ReturnType<typeof setTimeout> | undefined
+  let timeoutIdTrailing: ReturnType<typeof setTimeout> | undefined
 
   async function callback(...args: any): Promise<T> {
-    const callNow = immediate && timeoutID == null
+    const callNow = immediate && timeoutIdLeading == null
+
+    clearTimeout(timeoutIdLeading)
+    clearTimeout(timeoutIdTrailing)
+
     const promise = new Promise<T>((resolve, reject) => {
-      clearTimeout(timeoutID)
-
-      timeoutID = setTimeout(execute, callNow ? 0 : delay)
-
-      function execute(): void {
-        fn(...args)
-          .then(resolve)
-          .catch(reject)
-          .finally(() => {
-            timeoutID = undefined
-          })
+      if (callNow) {
+        timeoutIdLeading = setTimeout(() => {
+          fn(...args)
+            .then(resolve)
+            .catch(reject)
+        })
       }
+
+      timeoutIdTrailing = setTimeout(() => {
+        if (isFalse(immediate)) {
+          fn(...args)
+            .then(resolve)
+            .catch(reject)
+            .finally(() => {
+              timeoutIdTrailing = undefined
+            })
+        } else {
+          timeoutIdLeading = undefined
+          timeoutIdTrailing = undefined
+        }
+      }, delay)
     })
 
     return await promise
   }
 
   callback.cancel = () => {
-    clearTimeout(timeoutID)
-    timeoutID = undefined
+    clearTimeout(timeoutIdLeading)
+    clearTimeout(timeoutIdTrailing)
+    timeoutIdLeading = undefined
+    timeoutIdTrailing = undefined
   }
 
   return callback
