@@ -68,13 +68,18 @@ async def run_plan(
         end=plan.end,
     )
 
-    if plan.context_diff.has_changes:
+    if plan.context_diff.has_changes or plan.requires_backfill:
         batches = context.scheduler().batches()
         tasks = {snapshot.name: len(intervals) for snapshot, intervals in batches.items()}
 
         payload.backfills = [
             models.ContextEnvironmentBackfill(
                 model_name=interval.snapshot_name,
+                view_name=plan.context_diff.snapshots[
+                    interval.snapshot_name
+                ].qualified_view_name.for_environment(plan.environment.name)
+                if interval.snapshot_name in plan.context_diff.snapshots
+                else interval.snapshot_name,
                 interval=[
                     [to_ds(t) for t in make_inclusive(start, end)]
                     for start, end in interval.merged_intervals
